/*----------------------------------------------------------------------------
    ChucK Concurrent, On-the-fly Audio Programming Language
      Compiler and Virtual Machine

    Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
      http://chuck.cs.princeton.edu/
      http://soundlab.cs.princeton.edu/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// file: ulib_std.cpp
// desc: ...
//
// author: Ge Wang (gewang@cs.princeton.edu)
//         Perry R. Cook (prc@cs.princeton.edu)
// date: Spring 2004
//-----------------------------------------------------------------------------
#include "ulib_std.h"
#include <stdlib.h>
#include <time.h>




//-----------------------------------------------------------------------------
// name: libstd_query()
// desc: query entry point
//-----------------------------------------------------------------------------
DLL_QUERY libstd_query( Chuck_DL_Query * QUERY )
{
    QUERY->set_name( QUERY, "std" );
    
    // add abs
    QUERY->add_export( QUERY, "float", "abs", abs_impl, TRUE );
    QUERY->add_param( QUERY, "float", "value" );
    
    // add rand
    QUERY->add_export( QUERY, "float", "rand", rand_impl, TRUE );

    // add sgn
    QUERY->add_export( QUERY, "float", "sgn", sgn_impl, TRUE );
    QUERY->add_param( QUERY, "float", "value" );

    // add system
    QUERY->add_export( QUERY, "int", "system", system_impl, TRUE );
    QUERY->add_param( QUERY, "string", "cmd" );

    // add atoi
    QUERY->add_export( QUERY, "int", "atoi", atoi_impl, TRUE );
    QUERY->add_param( QUERY, "string", "value" );

    // add atof
    QUERY->add_export( QUERY, "float", "atof", atof_impl, TRUE );
    QUERY->add_param( QUERY, "string", "value" );

    // add getenv
    QUERY->add_export( QUERY, "string", "getenv", getenv_impl, TRUE );
    QUERY->add_param( QUERY, "string", "value" );

    // add setenv
    QUERY->add_export( QUERY, "int", "setenv", setenv_impl, TRUE );
    QUERY->add_param( QUERY, "string", "key" );
    QUERY->add_param( QUERY, "string", "value" );
    
    // seed the rand
    srand( time( NULL ) );
    
    return TRUE;
}


// abs
CK_DLL_FUNC( abs_impl )
{
    float v = *(float *)ARGS;
    RETURN->v_float = v >= 0.0f ? v : -v;
}

// fabs
CK_DLL_FUNC( fabs_impl )
{ }

// rand
CK_DLL_FUNC( rand_impl )
{
    RETURN->v_float = ( 2.0f * ::rand() / (float)RAND_MAX - 1.0f );
}

// sgn
CK_DLL_FUNC( sgn_impl )
{
    float v = *(float *)ARGS;
    RETURN->v_float = v == 0.0f ? 0.0f : ( v > 0.0f ? 1.0f : -1.0f );
}

// system
CK_DLL_FUNC( system_impl )
{
    char * cmd = *(char **)ARGS;
    RETURN->v_int = ::system( cmd );
}

// aoti
CK_DLL_FUNC( atoi_impl )
{
    char * v = *(char **)ARGS;
    RETURN->v_int = ::atoi( v );
}

// atof
CK_DLL_FUNC( atof_impl )
{
    char * v = *(char **)ARGS;
    RETURN->v_float = ::atof( v );
}

// getenv
CK_DLL_FUNC( getenv_impl )
{
    char * v = *(char **)ARGS;
    RETURN->v_string = ::getenv( v );
}

// setenv
CK_DLL_FUNC( setenv_impl )
{
    char * v1 = *(char **)ARGS;
    char * v2 = *( (char **)ARGS + 1 );
    RETURN->v_int = ::setenv( v1, v2, 1 );
}
